// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 30th July 2005
//
// Purpose:       Demo shows how to use the palette interpolation functions.
//
// Notes:         Graphics borrowed from Nessie (http://nessie.gbadev.org)
//
// -----------------------------------------------------------------------------
#include <hel.h>


// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
volatile u8 ATTR_MEM_IN_EWRAM g_NewFrame=TRUE;
TMapScrollInfo ATTR_MEM_IN_EWRAM g_MapInfos[1];


// -----------------------------------------------------------------------------
// External Data
// -----------------------------------------------------------------------------
extern const unsigned short world1_layer2_Map[880];
extern const unsigned short world1_layer2_Palette[16];
extern const unsigned char world1_layer2_Tiles[2336];
extern const unsigned short world1_layer2_evening_Palette[16];

// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void);



// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    hel_PadInit();
    ham_SetBgMode(0);
    
    // Load Background Palettes
    hel_PalBgLoad16((void*)world1_layer2_Palette, 2);
    
    // Init 16 Color Tilesets
    ham_bg[0].ti  = ham_InitTileSet((void*)&world1_layer2_Tiles, SIZEOF_16BIT(world1_layer2_Tiles), 0, 1);
    g_MapInfos[0] = hel_MapInit(0,  40, 22, 0, (void*)world1_layer2_Map);

    // Turn on Backgrounds
    ham_InitBg(0, TRUE, 0, FALSE);
    
    // Start VBL Interrupt
    ham_StartIntHandler(INT_TYPE_VBL, (void*)&VBLInterruptHandler);
    
    
    for(;;)
    {
        if(g_NewFrame)
        {
            hel_PadReadState();


            static u32 Step=0;
            
            if(hel_Pad.Held.Left)
              if(Step > 0) Step--;

            if(hel_Pad.Held.Right)
              if(Step < 31) Step++;
              
            // interpolate between palettes
            hel_PalBgInterpolate16
              (
                (void*)world1_layer2_Palette,         // Palette A
                (void*)world1_layer2_evening_Palette, // Palette B
                2,                                    // Palettebank (only neccessary in 16color mode)
                Step                                  // Interpolationstep (0..31)
              );
            
            g_NewFrame=FALSE;
        }
    }

    return 0;
}

// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler()
{
	g_NewFrame=TRUE;
}

