// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 10th July 2005
//
// Purpose:       Demo shows how to use interrupt functions, more exactly
//                how to obtain the type of a running interrupt.
//
// -----------------------------------------------------------------------------
#include <hel.h>


// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void SomeInterruptFunc(void);
void ShowInterruptType(void *InterruptProc);



// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    // Initialize HAMlib
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    ham_SetBgMode(0);
    ham_InitText(0);

    // Start an interrupt handler. In this case we start the VBL
    ham_StartIntHandler(INT_TYPE_VBL, (void*)SomeInterruptFunc);
    
    // Output what interrupt this function is assigned to!
    ShowInterruptType(&SomeInterruptFunc);
    
    // Loop
    for(;;);

    return 0;
}

// -----------------------------------------------------------------------------
// Simply an empty function, which we use as Interrupt Handler
// -----------------------------------------------------------------------------
void SomeInterruptFunc(void)
{
    // ...
}


// -----------------------------------------------------------------------------
// Show what Interrupt-Type the function specified by InterruptProc is
// -----------------------------------------------------------------------------
void ShowInterruptType(void *InterruptProc)
{
   switch(hel_IntrGetType(InterruptProc))
   {
      case INT_TYPE_VBL:
         ham_DrawText(1,1,"INT_TYPE_VBL");
         break;

      case INT_TYPE_HBL:
         ham_DrawText(1,1,"INT_TYPE_HBL");
         break;

      case INT_TYPE_VCNT:
         ham_DrawText(1,1,"INT_TYPE_VCNT");
         break;

      case INT_TYPE_TIM0:
         ham_DrawText(1,1,"INT_TYPE_TIM0");
         break;

      case INT_TYPE_TIM1:
         ham_DrawText(1,1,"INT_TYPE_TIM1");
         break;

      case INT_TYPE_TIM2:
         ham_DrawText(1,1,"INT_TYPE_TIM2");
         break;

      case INT_TYPE_TIM3:
         ham_DrawText(1,1,"INT_TYPE_TIM3");
         break;

      case INT_TYPE_SIO:
         ham_DrawText(1,1,"INT_TYPE_SIO");
         break;

      case INT_TYPE_DMA0:
         ham_DrawText(1,1,"INT_TYPE_DMA0");
         break;

      case INT_TYPE_DMA1:
         ham_DrawText(1,1,"INT_TYPE_DMA1");
         break;

      case INT_TYPE_DMA2:
         ham_DrawText(1,1,"INT_TYPE_DMA2");
         break;

      case INT_TYPE_DMA3:
         ham_DrawText(1,1,"INT_TYPE_DMA3");
         break;

      case INT_TYPE_KEY:
         ham_DrawText(1,1,"INT_TYPE_KEY");
         break;

      case INT_TYPE_CART:
         ham_DrawText(1,1,"INT_TYPE_CART");
         break;

      case 0xff:
         ham_DrawText(1,1,"No interrupt function", (u32)InterruptProc);
         ham_DrawText(1,2,"assigned to 0x%.8x", (u32)InterruptProc);
         break;
   }
}


